import { __rest } from "tslib";
import { loadPartialConfig } from "@babel/core";
import { setDefaultBy, WebpackConfigurer } from "@webpackery/core";
import { resolve } from "path";
import { TsconfigPathsPlugin } from "tsconfig-paths-webpack-plugin";
import { InjectEmptyExports } from "./inject-empty-exports";
import { findTsConfigFile, readTsConfigFile } from "./ts-utils";
export class BabelConfigurer extends WebpackConfigurer {

    prepare(props, context) {
        const { production } = context;

        props = Object.assign({}, props);

        props.options = Object.assign({ sourceMaps: production, envName: production ? "production" : "development", plugins: [], overrides: [] }, props.options);

        const plugins = props.options.plugins = Array.from(props.options.plugins);
        props.options.overrides = Array.from(props.options.overrides);
        const config = loadPartialConfig(Object.assign({ filename: resolve("./any.js"), caller: {
                name: "@webpackery/babel-configurer",
                supportsStaticESM: true,
                supportsDynamicImport: true,
            } }, props.options));
        setDefaultBy(props, "extensions", resolveDefaultExtensions, config);
        setDefaultBy(props, "test", extensionsToRegex, props.extensions);
        if (isTypeScript(config)) {
            const ts = require("typescript");
            const tsconfigPath = findTsConfigFile(ts, context.config.context);
            if (tsconfigPath) {
                const tsconfig = readTsConfigFile(ts, tsconfigPath);
                const { compilerOptions } = tsconfig;
                const { paths, preserveConstEnums } = compilerOptions;

                if (paths)
                    context.resolve.plugin(new TsconfigPathsPlugin({
                        extensions: props.extensions,
                        configFile: tsconfigPath,
                    }));
                const transform = preserveConstEnums ? "removeConst" : "constObject";
                plugins.push(InjectEmptyExports, ["babel-plugin-const-enum", { transform }]);
            }
        }
        return props;
    }

    configure(context, props) {
        const { extensions } = props, rule = __rest(props, ["extensions"]);
        context.module.rule(Object.assign({ loader: "babel-loader" }, rule));
        context.resolve.extension(...extensions);
    }
}
function extensionsToRegex(extensions) {
    if (extensions && extensions.length) {
        const group = extensions.map(x => x.slice(1)).join("|");
        const pattern = `\.(${group})$`;
        return new RegExp(pattern);
    }
}
function resolveDefaultExtensions(config) {
    const react = isReact(config);
    const typescript = isTypeScript(config);
    return [
        ".js", ".es6", ".es", ".mjs",
        react && ".jsx",
        typescript && [".ts", ".d.ts"],
        typescript && react && ".tsx",
    ].filter(Boolean).flat();
}
function isReact(config) {
    return hasFile(config, "@babel/preset-react");
}
function isTypeScript(config) {
    return hasFile(config, "@babel/preset-typescript");
}
function hasFile(config, preset) {
    const items = config.options.presets;
    return containsFileRequest(items, preset);
}
function containsFileRequest(items, preset) {
    if (items)
        return items.some(item => item.file && item.file.request === preset);
}
//# sourceMappingURL=babel-configurer.js.map
